using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.VisualStudio.Shell.Flavor;
using System.Runtime.InteropServices;
using Microsoft.VisualStudio.Shell.Interop;
using System.Drawing;
using EnvDTE;
using Microsoft.VisualStudio;

namespace Schaller.IconDemoPackage
{
	/// <summary>
	/// A project that is a flavor of an inner project.
	/// The default behavior of all methods is to delegate to the
	/// inner project. For any behavior you want to change, simply
	/// handle the request yourself, and delegate to the base class
	/// any case you don't want to handle
	/// </summary>
	[ComVisible( true )]
	[ClassInterface( ClassInterfaceType.None )]
	[Guid(GuidList.guidIconDemoProjectPkgString)]
	public class IconDemoProject : 
		FlavoredProjectBase
	{
		// internal so that our config can have easy access to it
		internal IconDemoPackage Package;

		private Icon m_icoFolderIconClosed;
		private Icon m_icoFolderIconOpened;

		private const string _SpecialFolderName = "MySpecialFolder";

		#region ... Handle Icons ...

		/// <summary>
		/// This function returns true only if the item id represent an item
		/// that is a special folder.
		/// </summary>
		/// <param name="itemID">itemID in the current Hierarchy representing the item asked about</param>
		/// <returns>true if we should replace the icon with our own</returns>
		private bool OverrideThisItemIcon( uint itemID )
		{
			// Don't override the root (project node) as it is never code
			if (itemID == VSConstants.VSITEMID_ROOT)
				return false;

			try
			{
				// Get DTE Item
				object objDteObject = null;
				((IVsHierarchy) this).GetProperty( itemID, (int) __VSHPROPID.VSHPROPID_ExtObject, out objDteObject );
				if (objDteObject != null)
				{
					if (objDteObject is EnvDTE.ProjectItem)
					{
						EnvDTE.ProjectItem objDteItem = (EnvDTE.ProjectItem) objDteObject;
						return (objDteItem.Name == _SpecialFolderName);
					}
				}
			}
			catch
			{
			}
			return false;
		}

		#endregion

		#region IVsAggregatableProject Members

		/// <summary>
		/// This should first QI for (and keep a reference to) each interface we plan to call on the inner project
		/// and then call the base implementation to do the rest. Because the base implementation
		/// already keep a reference to the interfaces it override, we don't need to QI for those.
		/// </summary>
		protected override void SetInnerProject( IntPtr innerIUnknown )
		{
			object inner = null;

			inner = Marshal.GetObjectForIUnknown( innerIUnknown );

			// Ensure we have a service provider as this is required for menu items to work
			if (this.serviceProvider == null)
				this.serviceProvider = (System.IServiceProvider) this.Package;

			// Now let the base implementation set the inner object
			base.SetInnerProject( innerIUnknown );

		}

		protected override void InitializeForOuter( string fileName, string location, string name, uint flags, ref Guid guidProject, out bool cancel )
		{
			base.InitializeForOuter( fileName, location, name, flags, ref guidProject, out cancel );

			//
			// Load folder icons
			//
			m_icoFolderIconClosed = Resources.ClosedFolder;
			m_icoFolderIconOpened = Resources.OpenFolder;
		}

		#endregion

		#region ... IVsHierarchy ...

		/// <summary>
		/// Overriding this method as we want to filter some of the properties
		/// For properties we don't want to handle, let the base class delegate
		/// to the inner project
		/// </summary>
		protected override int GetProperty( uint itemId, int propId, out object property )
		{
			switch ( propId )
			{
				case (int) __VSHPROPID.VSHPROPID_IconImgList:
				case (int) __VSHPROPID.VSHPROPID_IconIndex:
				case (int) __VSHPROPID.VSHPROPID_OpenFolderIconIndex:
					{
						if (OverrideThisItemIcon( itemId ))
						{
							property = null;
							return VSConstants.E_NOTIMPL;
						}
						break;
					}
				case (int) __VSHPROPID.VSHPROPID_IconHandle:
					{
						if ( m_icoFolderIconClosed != null && OverrideThisItemIcon( itemId ) )
						{
							property = m_icoFolderIconClosed.Handle;
							return VSConstants.S_OK;
						}
						break;
					}
				case (int) __VSHPROPID.VSHPROPID_OpenFolderIconHandle:
					{
						if ( m_icoFolderIconOpened != null && OverrideThisItemIcon( itemId ) )
						{
							property = m_icoFolderIconOpened.Handle;
							return VSConstants.S_OK;
						}
						break;
					}
			}
			return base.GetProperty( itemId, propId, out property );
		}

		#endregion
	}
}
